package de.intevation.flys.importer;

import de.intevation.flys.model.Wst;
import de.intevation.flys.model.WstColumnValue;
import de.intevation.flys.model.River;

import de.intevation.flys.backend.SessionFactoryProvider;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Query;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ImportWst
{
    private static Logger logger = Logger.getLogger(ImportWst.class);

    protected String description;

    protected Integer kind;

    //protected Map<String, ImportWstColumn> columns;
    protected List<ImportWstColumn> columns;

    protected Map<WstColumnValueKey, WstColumnValue> wstCache;

    protected Wst peer;

    public ImportWst() {
        kind = 0;
        columns = new ArrayList<ImportWstColumn>();

        // This cache is used to avoid db queries for each WstColumnValue later
        wstCache = getWstColumnValueCache();
    }

    public ImportWst(String description) {
        this();
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    public Integer getKind() {
        return kind;
    }

    public void setKind(Integer kind) {
        this.kind = kind;
    }


    public void setDescription(String description) {
        this.description = description;
    }

    public void setNumberColumns(int numColumns) {
        for (int i = 0; i < numColumns; ++i) {
            columns.add(new ImportWstColumn(this, null, null, i, wstCache));
        }
    }

    public int getNumberColumns() {
        return columns.size();
    }

    public ImportWstColumn getColumn(int index) {
        return columns.get(index);
    }

    public void storeDependencies(River river) {
        Wst wst = getPeer(river);

        for (ImportWstColumn column: columns) {
            column.storeDependencies(river);
        }
    }


    public Map<WstColumnValueKey, WstColumnValue> getWstColumnValueCache() {
        SessionFactory sessionFactory =
            SessionFactoryProvider.createSessionFactory();

        Session session = sessionFactory.openSession();

        Importer.sessionHolder.set(session);

        try {
            Query query = session.createQuery("from WstColumnValue");

            List<WstColumnValue> values = query.list();
            Map<WstColumnValueKey, WstColumnValue> cache =
                new HashMap<WstColumnValueKey, WstColumnValue>();

            for (WstColumnValue value: values) {
                cache.put(
                    new WstColumnValueKey(value),
                    value);
            }

            logger.info("++++++++++++++++++++++++++++++++++++++++++++++++++++");
            logger.info(cache.size() + " WstColumnValue objects in WST cache.");
            logger.info("++++++++++++++++++++++++++++++++++++++++++++++++++++");

            return cache;
        }
        finally {
            session.close();
            Importer.sessionHolder.remove();
        }
    }

    public Wst getPeer(River river) {
        if (peer == null) {
            Session session = Importer.sessionHolder.get();
            Query query = session.createQuery(
                "from Wst where " +
                "river=:river and description=:description and kind=:kind");
            query.setParameter("river",       river);
            query.setParameter("description", description);
            query.setParameter("kind",        kind);
            List<Wst> wsts = query.list();
            if (wsts.isEmpty()) {
                peer = new Wst(river, description, kind);
                session.save(peer);
            }
            else {
                peer = wsts.get(0);
            }

        }
        return peer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
